@@ -3,7 +3,7 @@ import sys, os.path
 
 builtin_repr = repr
 
-repr = py.builtin._tryimport('repr', 'reprlib')
+reprlib = py.builtin._tryimport('repr', 'reprlib')
 
 class Code(object):
     """ wrapper around Python code objects """
@@ -23,64 +23,14 @@ class Code(object):
     def __ne__(self, other):
         return not self == other
 
-    def new(self, rec=False, **kwargs): 
-        """ return new code object with modified attributes. 
-            if rec-cursive is true then dive into code 
-            objects contained in co_consts. 
-        """ 
-        if sys.platform.startswith("java"):
-            # XXX jython does not support the below co_filename hack
-            return self.raw 
-        names = [x for x in dir(self.raw) if x[:3] == 'co_']
-        for name in kwargs: 
-            if name not in names: 
-                raise TypeError("unknown code attribute: %r" %(name, ))
-        if rec and hasattr(self.raw, 'co_consts'):  # jython 
-            newconstlist = []
-            co = self.raw
-            cotype = type(co)
-            for c in co.co_consts:
-                if isinstance(c, cotype):
-                    c = self.__class__(c).new(rec=True, **kwargs) 
-                newconstlist.append(c)
-            return self.new(rec=False, co_consts=tuple(newconstlist), **kwargs) 
-        for name in names:
-            if name not in kwargs:
-                kwargs[name] = getattr(self.raw, name)
-        arglist = [
-                 kwargs['co_argcount'],
-                 kwargs['co_nlocals'],
-                 kwargs.get('co_stacksize', 0), # jython
-                 kwargs.get('co_flags', 0), # jython
-                 kwargs.get('co_code', ''), # jython
-                 kwargs.get('co_consts', ()), # jython
-                 kwargs.get('co_names', []), # 
-                 kwargs['co_varnames'],
-                 kwargs['co_filename'],
-                 kwargs['co_name'],
-                 kwargs['co_firstlineno'],
-                 kwargs.get('co_lnotab', ''), #jython
-                 kwargs.get('co_freevars', None), #jython
-                 kwargs.get('co_cellvars', None), # jython
-        ]
-        if sys.version_info >= (3,0):
-            arglist.insert(1, kwargs['co_kwonlyargcount'])
-            return self.raw.__class__(*arglist)
-        else:
-            return py.std.new.code(*arglist)
-
     def path(self):
         """ return a path object pointing to source code"""
-        fn = self.raw.co_filename 
-        try:
-            return fn.__path__
-        except AttributeError:
-            p = py.path.local(self.raw.co_filename)
-            if not p.check():
-                # XXX maybe try harder like the weird logic 
-                # in the standard lib [linecache.updatecache] does? 
-                p = self.raw.co_filename
-            return p
+        p = py.path.local(self.raw.co_filename)
+        if not p.check():
+            # XXX maybe try harder like the weird logic 
+            # in the standard lib [linecache.updatecache] does? 
+            p = self.raw.co_filename
+        return p
                 
     path = property(path, None, None, "path of this code object")
 
@@ -147,7 +97,7 @@ class Frame(object):
     def repr(self, object):
         """ return a 'safe' (non-recursive, one-line) string repr for 'object'
         """
-        return safe_repr(object)
+        return py.io.saferepr(object)
 
     def is_true(self, object):
         return object
@@ -199,9 +149,8 @@ class TracebackEntry(object):
         """Reinterpret the failing statement and returns a detailed information
            about what operations are performed."""
         if self.exprinfo is None:
-            from py._code import assertion 
             source = str(self.statement).strip()
-            x = assertion.interpret(source, self.frame, should_fail=True)
+            x = py.code._reinterpret(source, self.frame, should_fail=True)
             if not isinstance(x, str):
                 raise TypeError("interpret returned non-string %r" % (x,))
             self.exprinfo = x 
@@ -417,6 +366,12 @@ class ExceptionInfo(object):
         loc = ReprFileLocation(entry.path, entry.lineno + 1, self.exconly())
         return str(loc)
 
+    def __unicode__(self):
+        entry = self.traceback[-1]
+        loc = ReprFileLocation(entry.path, entry.lineno + 1, self.exconly())
+        return unicode(loc)
+        
+
 class FormattedExcinfo(object):
     """ presenting information about failing Functions and Generators. """ 
     # for traceback entries 
@@ -452,7 +407,7 @@ class FormattedExcinfo(object):
         return source
     
     def _saferepr(self, obj):
-        return safe_repr(obj)
+        return py.io.saferepr(obj)
 
     def repr_args(self, entry):
         if self.funcargs:
@@ -461,7 +416,7 @@ class FormattedExcinfo(object):
                 args.append((argname, self._saferepr(argvalue)))
             return ReprFuncArgs(args)
 
-    def get_source(self, source, line_index=-1, excinfo=None):
+    def get_source(self, source, line_index=-1, excinfo=None, short=False):
         """ return formatted and marked up source lines. """
         lines = []
         if source is None:
@@ -473,6 +428,8 @@ class FormattedExcinfo(object):
             if i == line_index:
                 prefix = self.flow_marker + "   "
             else:
+                if short:
+                    continue
                 prefix = "    "
             line = prefix + source[i]
             lines.append(line)
@@ -504,7 +461,7 @@ class FormattedExcinfo(object):
                     lines.append("__builtins__ = <builtins>")
                 else:
                     # This formatting could all be handled by the
-                    # _repr() function, which is only repr.Repr in
+                    # _repr() function, which is only reprlib.Repr in
                     # disguise, so is very configurable.
                     str_repr = self._saferepr(value)
                     #if len(str_repr) < 70 or not isinstance(value,
@@ -523,27 +480,30 @@ class FormattedExcinfo(object):
             source = py.code.Source("???")
             line_index = 0
         else:
-            line_index = entry.lineno - entry.getfirstlinesource()
+            # entry.getfirstlinesource() can be -1, should be 0 on jython
+            line_index = entry.lineno - max(entry.getfirstlinesource(), 0)
 
         lines = []
-        if self.style == "long":
-            reprargs = self.repr_args(entry) 
-            lines.extend(self.get_source(source, line_index, excinfo))
-            message = excinfo and excinfo.typename or ""
+        if self.style in ("short", "long"):
+            short = self.style == "short"
+            reprargs = None
+            if not short:
+                reprargs = self.repr_args(entry) 
+            s = self.get_source(source, line_index, excinfo, short=short)
+            lines.extend(s)
+            if short:
+                message = "in %s" %(entry.name)
+            else:
+                message = excinfo and excinfo.typename or ""
             path = self._makepath(entry.path)
             filelocrepr = ReprFileLocation(path, entry.lineno+1, message)
-            localsrepr =  self.repr_locals(entry.locals)
-            return ReprEntry(lines, reprargs, localsrepr, filelocrepr)
-        else: 
-            if self.style == "short":
-                line = source[line_index].lstrip()
-                basename = os.path.basename(entry.frame.code.filename)
-                lines.append('  File "%s", line %d, in %s' % (
-                    basename, entry.lineno+1, entry.name))
-                lines.append("    " + line) 
-            if excinfo: 
-                lines.extend(self.get_exconly(excinfo, indent=4))
-            return ReprEntry(lines, None, None, None)
+            localsrepr = None
+            if not short:
+                localsrepr =  self.repr_locals(entry.locals)
+            return ReprEntry(lines, reprargs, localsrepr, filelocrepr, short)
+        if excinfo: 
+            lines.extend(self.get_exconly(excinfo, indent=4))
+        return ReprEntry(lines, None, None, None, False)
 
     def _makepath(self, path):
         if not self.abspath:
@@ -578,13 +538,29 @@ class FormattedExcinfo(object):
 
 class TerminalRepr:
     def __str__(self):
-        tw = py.io.TerminalWriter(stringio=True)
+        s = self.__unicode__()
+        if sys.version_info[0] < 3:
+            s = s.encode('utf-8')
+        return s
+
+    def __unicode__(self):
+        l = []
+        tw = py.io.TerminalWriter(l.append)
         self.toterminal(tw)
-        return tw.stringio.getvalue().strip()
+        l = map(unicode_or_repr, l)
+        return "".join(l).strip()
 
     def __repr__(self):
         return "<%s instance at %0x>" %(self.__class__, id(self))
 
+def unicode_or_repr(obj):
+    try:
+        return py.builtin._totext(obj)
+    except KeyboardInterrupt:
+        raise
+    except Exception:
+        return "<print-error: %r>" % py.io.saferepr(obj)
+
 class ReprExceptionInfo(TerminalRepr):
     def __init__(self, reprtraceback, reprcrash):
         self.reprtraceback = reprtraceback
@@ -623,13 +599,21 @@ class ReprTraceback(TerminalRepr):
 class ReprEntry(TerminalRepr):
     localssep = "_ "
 
-    def __init__(self, lines, reprfuncargs, reprlocals, filelocrepr):
+    def __init__(self, lines, reprfuncargs, reprlocals, filelocrepr, short):
         self.lines = lines
         self.reprfuncargs = reprfuncargs
         self.reprlocals = reprlocals 
         self.reprfileloc = filelocrepr
+        self.short = short
 
     def toterminal(self, tw):
+        if self.short:
+            self.reprfileloc.toterminal(tw)
+            for line in self.lines:
+                red = line.startswith("E   ") 
+                tw.line(line, bold=True, red=red)
+            #tw.line("")
+            return
         if self.reprfuncargs:
             self.reprfuncargs.toterminal(tw)
         for line in self.lines:
@@ -695,48 +679,6 @@ class ReprFuncArgs(TerminalRepr):
 
 
 
-class SafeRepr(repr.Repr):
-    """ subclass of repr.Repr that limits the resulting size of repr() 
-        and includes information on exceptions raised during the call. 
-    """ 
-    def __init__(self, *args, **kwargs):
-        repr.Repr.__init__(self, *args, **kwargs)
-        self.maxstring = 240   # 3 * 80 chars
-        self.maxother = 160    # 2 * 80 chars
-
-    def repr(self, x):
-        return self._callhelper(repr.Repr.repr, self, x)
-
-    def repr_instance(self, x, level):
-        return self._callhelper(builtin_repr, x)
-        
-    def _callhelper(self, call, x, *args):
-        try:
-            # Try the vanilla repr and make sure that the result is a string
-            s = call(x, *args)
-        except (KeyboardInterrupt, MemoryError, SystemExit):
-            raise
-        except:
-            cls, e, tb = sys.exc_info()
-            try:
-                exc_name = cls.__name__
-            except:
-                exc_name = 'unknown'
-            try:
-                exc_info = str(e)
-            except:
-                exc_info = 'unknown'
-            return '<[%s("%s") raised in repr()] %s object at 0x%x>' % (
-                exc_name, exc_info, x.__class__.__name__, id(x))
-        else:
-            if len(s) > self.maxstring:
-                i = max(0, (self.maxstring-3)//2)
-                j = max(0, self.maxstring-3-i)
-                s = s[:i] + '...' + s[len(s)-j:]
-            return s
-
-safe_repr = SafeRepr().repr
-
 oldbuiltins = {}
 
 def patch_builtins(assertion=True, compile=True):
