@@ -6,16 +6,22 @@ This is a good source for looking at the various reporting hooks.
 import py
 import sys
 
+optionalhook = py.test.mark.optionalhook
+
 def pytest_addoption(parser):
     group = parser.getgroup("terminal reporting", "reporting", after="general")
     group._addoption('-v', '--verbose', action="count", 
                dest="verbose", default=0, help="increase verbosity."),
+    group._addoption('-r',
+         action="store", dest="reportchars", default=None, metavar="chars",
+         help="show extra test summary info as specified by chars (f)ailed, "
+              "(s)skipped, (x)failed, (X)passed.")
     group._addoption('-l', '--showlocals',
-               action="store_true", dest="showlocals", default=False,
-               help="show locals in tracebacks (disabled by default).")
-    group.addoption('--report',
-               action="store", dest="report", default=None, metavar="opts",
-               help="show more info, valid: skipped,xfailed")
+         action="store_true", dest="showlocals", default=False,
+         help="show locals in tracebacks (disabled by default).")
+    group._addoption('--report',
+         action="store", dest="report", default=None, metavar="opts",
+         help="(deprecated, use -r)")
     group._addoption('--tb', metavar="style", 
                action="store", dest="tbstyle", default='long',
                type="choice", choices=['long', 'short', 'no', 'line'],
@@ -45,17 +51,25 @@ def pytest_configure(config):
                 setattr(reporter._tw, name, getattr(config, attr))
         config.pluginmanager.register(reporter, 'terminalreporter')
 
-def getreportopt(optvalue):
-    d = {}
+def getreportopt(config):
+    reportopts = ""
+    optvalue = config.getvalue("report")
     if optvalue:
-        for setting in optvalue.split(","):
-            setting = setting.strip()
-            val = True
-            if setting.startswith("no"):
-                val = False
-                setting = setting[2:]
-            d[setting] = val
-    return d
+        py.builtin.print_("DEPRECATED: use -r instead of --report option.", 
+            file=py.std.sys.stderr)
+        if optvalue:
+            for setting in optvalue.split(","):
+                setting = setting.strip()
+                if setting == "skipped":
+                    reportopts += "s"
+                elif setting == "xfailed":
+                    reportopts += "x"
+    reportchars = config.getvalue("reportchars")
+    if reportchars:
+        for char in reportchars:
+            if char not in reportopts:
+                reportopts += char
+    return reportopts
 
 class TerminalReporter:
     def __init__(self, config, file=None):
@@ -67,10 +81,11 @@ class TerminalReporter:
         self._tw = py.io.TerminalWriter(file)
         self.currentfspath = None 
         self.gateway2info = {}
-        self._reportopt = getreportopt(config.getvalue('report'))
+        self.reportchars = getreportopt(config)
 
-    def hasopt(self, name):
-        return self._reportopt.get(name, False)
+    def hasopt(self, char):
+        char = {'xfailed': 'x', 'skipped': 's'}.get(char,char)
+        return char in self.reportchars
 
     def write_fspath_result(self, fspath, res):
         fspath = self.curdir.bestrelpath(fspath)
@@ -126,10 +141,33 @@ class TerminalReporter:
         else: 
             return "???", dict(red=True)
 
+    def gettestid(self, item, relative=True):
+        fspath = item.fspath
+        chain = [x for x in item.listchain() if x.fspath == fspath]
+        chain = chain[1:]
+        names = [x.name for x in chain if x.name != "()"]
+        path = item.fspath
+        if relative:
+            relpath = path.relto(self.curdir)
+            if relpath:
+                path = relpath
+        names.insert(0, str(path))
+        return "::".join(names)
+
+
     def pytest_internalerror(self, excrepr):
         for line in str(excrepr).split("\n"):
             self.write_line("INTERNALERROR> " + line)
 
+    def pytest_plugin_registered(self, plugin):
+        if self.config.option.traceconfig: 
+            msg = "PLUGIN registered: %s" %(plugin,)
+            # XXX this event may happen during setup/teardown time 
+            #     which unfortunately captures our output here 
+            #     which garbles our output if we use self.write_line 
+            self.write_line(msg)
+
+    @optionalhook
     def pytest_gwmanage_newgateway(self, gateway, platinfo):
         #self.write_line("%s instantiated gateway from spec %r" %(gateway.id, gateway.spec._spec))
         d = {}
@@ -149,30 +187,37 @@ class TerminalReporter:
         self.write_line(infoline)
         self.gateway2info[gateway] = infoline
 
-    def pytest_plugin_registered(self, plugin):
-        if self.config.option.traceconfig: 
-            msg = "PLUGIN registered: %s" %(plugin,)
-            # XXX this event may happen during setup/teardown time 
-            #     which unfortunately captures our output here 
-            #     which garbles our output if we use self.write_line 
-            self.write_line(msg)
-
+    @optionalhook
     def pytest_testnodeready(self, node):
         self.write_line("[%s] txnode ready to receive tests" %(node.gateway.id,))
 
+    @optionalhook
     def pytest_testnodedown(self, node, error):
         if error:
             self.write_line("[%s] node down, error: %s" %(node.gateway.id, error))
 
+    @optionalhook
+    def pytest_rescheduleitems(self, items):
+        if self.config.option.debug:
+            self.write_sep("!", "RESCHEDULING %s " %(items,))
+
+    @optionalhook
+    def pytest_looponfailinfo(self, failreports, rootdirs):
+        if failreports:
+            self.write_sep("#", "LOOPONFAILING", red=True)
+            for report in failreports:
+                loc = self._getcrashline(report)
+                self.write_line(loc, red=True)
+        self.write_sep("#", "waiting for changes")
+        for rootdir in rootdirs:
+            self.write_line("### Watching:   %s" %(rootdir,), bold=True)
+
+
     def pytest_trace(self, category, msg):
         if self.config.option.debug or \
            self.config.option.traceconfig and category.find("config") != -1:
             self.write_line("[%s] %s" %(category, msg))
 
-    def pytest_rescheduleitems(self, items):
-        if self.config.option.debug:
-            self.write_sep("!", "RESCHEDULING %s " %(items,))
-
     def pytest_deselected(self, items):
         self.stats.setdefault('deselected', []).append(items)
 
@@ -239,7 +284,7 @@ class TerminalReporter:
         self._sessionstarttime = py.std.time.time()
 
         verinfo = ".".join(map(str, sys.version_info[:3]))
-        msg = "python: platform %s -- Python %s" % (sys.platform, verinfo)
+        msg = "platform %s -- Python %s" % (sys.platform, verinfo)
         msg += " -- pytest-%s" % (py.__version__)
         if self.config.option.verbose or self.config.option.debug or getattr(self.config.option, 'pastebin', None):
             msg += " -- " + str(sys.executable)
@@ -267,22 +312,14 @@ class TerminalReporter:
         self._keyboardinterrupt_memo = excinfo.getrepr(funcargs=True)
 
     def _report_keyboardinterrupt(self):
-        self.write_sep("!", "KEYBOARD INTERRUPT")
         excrepr = self._keyboardinterrupt_memo
-        if self.config.option.verbose:
-            excrepr.toterminal(self._tw)
-        else:
-            excrepr.reprcrash.toterminal(self._tw)
-
-    def pytest_looponfailinfo(self, failreports, rootdirs):
-        if failreports:
-            self.write_sep("#", "LOOPONFAILING", red=True)
-            for report in failreports:
-                loc = self._getcrashline(report)
-                self.write_line(loc, red=True)
-        self.write_sep("#", "waiting for changes")
-        for rootdir in rootdirs:
-            self.write_line("### Watching:   %s" %(rootdir,), bold=True)
+        msg = excrepr.reprcrash.message
+        self.write_sep("!", msg)
+        if "KeyboardInterrupt" in msg:
+            if self.config.getvalue("fulltrace"):
+                excrepr.toterminal(self._tw)
+            else:
+                excrepr.reprcrash.toterminal(self._tw)
 
     def _getcrashline(self, report):
         try:
@@ -497,7 +534,7 @@ class ShowFuncargSession(Session):
     def getlocation(self, function):
         import inspect
         fn = py.path.local(inspect.getfile(function))
-        lineno = function.func_code.co_firstlineno
+        lineno = py.builtin._getcode(function).co_firstlineno
         if fn.relto(self.fspath):
             fn = fn.relto(self.fspath)
         return "%s:%d" %(fn, lineno+1)
