@@ -4,6 +4,13 @@ from py.path import local
 from testing.path import common
 
 failsonjython = py.test.mark.xfail("sys.platform.startswith('java')")
+failsonjywin32 = py.test.mark.xfail("sys.platform.startswith('java') "
+        "and getattr(os, '_name', None) == 'nt'")
+win32only = py.test.mark.skipif(
+        "not (sys.platform == 'win32' or getattr(os, '_name', None) == 'nt')")
+skiponwin32 = py.test.mark.skipif(
+        "sys.platform == 'win32' or getattr(os, '_name', None) == 'nt'")
+
 
 def pytest_funcarg__path1(request):
     def setup():
@@ -85,6 +92,7 @@ class TestLocalPath(common.CommonFSTests):
         finally:
             f.close()
 
+    @failsonjywin32
     def test_setmtime(self):
         import tempfile
         import time
@@ -188,12 +196,12 @@ class TestLocalPath(common.CommonFSTests):
         p2 = tmpdir.ensure("b","2")
         p3 = tmpdir.ensure("breadth")
         l = list(tmpdir.visit(lambda x: x.check(file=1)))
-        assert l[0] == p1
-        assert l[1] == p2 
-        assert l[2] == p3
+        assert len(l) == 3
+        # check that breadth comes last
+        assert l[2] == p3 
 
 class TestExecutionOnWindows:
-    pytestmark = py.test.mark.skipif("sys.platform != 'win32'")
+    pytestmark = win32only
 
     def test_sysfind(self):
         x = py.path.local.sysfind('cmd')
@@ -201,7 +209,7 @@ class TestExecutionOnWindows:
         assert py.path.local.sysfind('jaksdkasldqwe') is None
 
 class TestExecution:
-    pytestmark = py.test.mark.skipif("sys.platform == 'win32'")
+    pytestmark = skiponwin32
 
     def test_sysfind(self):
         x = py.path.local.sysfind('test')
@@ -347,6 +355,25 @@ def test_pypkgdir(tmpdir):
     assert pkg.pypkgpath() == pkg
     assert pkg.join('subdir', '__init__.py').pypkgpath() == pkg
 
+def test_pypkgdir_unimportable(tmpdir):
+    pkg = tmpdir.ensure('pkg1-1', dir=1) # unimportable
+    pkg.ensure("__init__.py")
+    subdir = pkg.ensure("subdir/__init__.py").dirpath()
+    assert subdir.pypkgpath() == subdir
+    assert subdir.ensure("xyz.py").pypkgpath() == subdir
+    assert not pkg.pypkgpath() 
+
+def test_isimportable():
+    from py._path.local import isimportable
+    assert not isimportable("")
+    assert isimportable("x")
+    assert isimportable("x1")
+    assert isimportable("x_1")
+    assert isimportable("_")
+    assert isimportable("_1")
+    assert not isimportable("x-1")
+    assert not isimportable("x:1")
+
 def test_homedir():
     homedir = py.path.local._gethomedir()
     assert homedir.check(dir=1)
@@ -357,7 +384,7 @@ def test_samefile(tmpdir):
     assert p.samefile(p) 
 
 class TestWINLocalPath:
-    pytestmark = py.test.mark.skipif("sys.platform != 'win32'")
+    pytestmark = win32only
 
     def test_owner_group_not_implemented(self, path1):
         py.test.raises(NotImplementedError, "path1.stat().owner")
@@ -406,7 +433,7 @@ class TestWINLocalPath:
             old.chdir()    
 
 class TestPOSIXLocalPath:
-    pytestmark = py.test.mark.skipif("sys.platform == 'win32'")
+    pytestmark = skiponwin32
 
     def test_hardlink(self, tmpdir):
         linkpath = tmpdir.join('test')
@@ -499,7 +526,9 @@ class TestPOSIXLocalPath:
         # we could wait here but timer resolution is very
         # system dependent 
         path.read()
+        time.sleep(0.01) 
         atime2 = path.atime()
+        time.sleep(0.01)
         duration = time.time() - now
         assert (atime2-atime1) <= duration
 
